/**
 * ManageIngredientsWindow allows the user to edit preexisting ingredients in the
 * database.
 * 
 * @author Nathan
 */

package eShopper;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.sql.*;

/**
 * This window allows the user to edit the properties of the
 * ingredients in the database.
 * 
 * @author Nathan
 *
 */
public class ManageIngredientsWindow extends IngredientWindow  {
	
	ManageIngredientsWindow(eShopperList list){
		super(list);
		this.list = list;
		this.database = list.getDB();
		
		setSize(350, 242);
		setTitle("eShopper: Manage Ingredients");
		
		//
		// edit namePanel
		//
		
		namePanel.setMinimumSize( new Dimension(320, 65) );
		namePanel.setMaximumSize( new Dimension(320, 65) );
		namePanel.setPreferredSize( new Dimension(320, 65) );
		namePanel.setLayout(new BoxLayout(namePanel, BoxLayout.Y_AXIS));
		JPanel nameSubPanel = new JPanel();
		nameSubPanel.add(new JLabel("Select one: "));
		
		//populate ingredientBox from database
		Vector ingredientVector = new Vector();
			try{
				ResultSet rs = database.query("SELECT ID, Name FROM Ingredient ORDER BY Name;");
				while (rs.next()){
					ingredientVector.add(new Ingredient(rs.getInt(1), rs.getString(2)));
				}
			}
			catch (SQLException se) {
				System.out.println("Unable to get ingredients from database. Error:");
				se.printStackTrace();
			}
		
		ingredientBox = new JComboBox(ingredientVector);
		ingredientBox.addActionListener(this);
		ingredientBox.setActionCommand("ingredientBox");
		
		nameSubPanel.add(ingredientBox);
		namePanel.add(nameSubPanel);
		namePanel.add( Box.createRigidArea(new Dimension(0,5)) );
		
		// The buttons
		JPanel buttonSubPanel = new JPanel();
		buttonSubPanel.setLayout(new BoxLayout(buttonSubPanel, BoxLayout.X_AXIS));
		buttonSubPanel.add(Box.createHorizontalGlue());
		
		// Delete button
		JButton deleteIngredientButton = new JButton("Delete");
		deleteIngredientButton.addActionListener(this);
		deleteIngredientButton.setActionCommand("delete");
		deleteIngredientButton.setMnemonic(KeyEvent.VK_D);
		buttonSubPanel.add(deleteIngredientButton);
		buttonSubPanel.add( Box.createRigidArea(new Dimension(5,0)) );
				
		// Rename button
		JButton renameIngredientButton = new JButton("Rename");
		renameIngredientButton.addActionListener(this);
		renameIngredientButton.setActionCommand("rename");
		renameIngredientButton.setMnemonic(KeyEvent.VK_R);
		buttonSubPanel.add(renameIngredientButton);
		buttonSubPanel.add( Box.createRigidArea(new Dimension(5,0)) );
		
		// Create button
		JButton addIngredientButton = new JButton("Create New...");
		addIngredientButton.addActionListener(this);
		addIngredientButton.setActionCommand("Create New");
		addIngredientButton.setMnemonic(KeyEvent.VK_C);
		buttonSubPanel.add(addIngredientButton);
		namePanel.add( buttonSubPanel );
		
		// fill description field from database
		getDescription((Ingredient)ingredientBox.getSelectedItem());
		
		
		mainPanel.add(namePanel);
		mainPanel.add( Box.createRigidArea(new Dimension(0,5)) );
		mainPanel.add(descriptionPanel);
		mainPanel.add( Box.createRigidArea(new Dimension(0,17)) );
		mainPanel.add(buttonPanel);
		getContentPane().add(mainPanel);
	}
	
	/**
	 * This fills the description field from database.
	 */
	public void getDescription(Ingredient ingredient){
		try{
			ResultSet rs = database.query("SELECT Description FROM Ingredient WHERE ID = '" + ingredient.getID() + "';");
			rs.next();
			descArea.setText(rs.getString(1));
		}
		catch (SQLException se) {
			System.out.println("Unable to get description from database. Error:");
			se.printStackTrace();
			descArea.setText("");
		}
		descArea.repaint();
	}

	/**
	 * This updates an ingredient's description in the database.
	 * It takes the value in the Description Box, and inserts it into
	 * the description field associated with the ingredient selected in
	 * the combobox.
	 */
	public void updateDatabase(){
		try{
			database.update("UPDATE Ingredient SET Description = '" + descArea.getText().replaceAll("'", "''") + "' WHERE Name = '" + ingredientBox.getSelectedItem().toString().replaceAll("'", "''") + "';");
			JOptionPane.showMessageDialog(this, "Database updated");
		}
		catch (SQLException se) {
			System.out.println("Unable to write to database. Error:");
			se.printStackTrace();
		}
	}

	/**
	 * This deletes an ingredient both from the comboBox
	 * in the window, and from the database.  It prompts the user
	 * for confirmation before deleting.  If the user says no or
	 * cancels the confirmation dialog, nothing is deleted.
	 */
	private void deleteIngredient(){
		int n = JOptionPane.showConfirmDialog(
                mainPanel, "Are you sure you want to delete ingredient " + ingredientBox.getSelectedItem() + "?\n" +
				"This will also delete it from all shopping lists.",
                "Delete Confirmation",
                JOptionPane.YES_NO_OPTION);
        if (n == JOptionPane.YES_OPTION) {
            // First, check to see if any recipes depend on this ingredient.
        	try{
        		String q = "SELECT Recipe.Name FROM Recipe, Ingredient, Has " +
							"WHERE Recipe.ID = Has.RecipeID " +
							"AND Has.IngredientID = Ingredient.ID " +
							"AND Ingredient.ID = '" + ((Ingredient)ingredientBox.getSelectedItem()).getID() + "'";
        		ResultSet rs = database.query(q);
        		
        		// There is a recipe that requires this ingredient.
        		if (rs.next()) {
        			JOptionPane.showMessageDialog(mainPanel,
        				    "Error: " + ingredientBox.getSelectedItem() + " is required by a recipe.  It cannot be deleted.",
        				    "eShopper Error",
        				    JOptionPane.ERROR_MESSAGE);
        			return;
        		}
        	}
        	catch (SQLException se) {
        		System.out.println("Error: ManageIngredientsWindow::deleteIngredient (1)");
    			se.printStackTrace();
    		}
        	
            // If not, delete from the database.
            try{
            	int ingredientID = ((Ingredient)ingredientBox.getSelectedItem()).getID();
            		
                // Delete from all shopping lists.
                String q = "DELETE FROM IngredientList WHERE IngredientID = '" + ingredientID + "'";
                database.update(q);
                	
                // Delete from Ingredients database.
                q = "DELETE FROM Ingredient WHERE ID = '" + ingredientID + "'";
                database.update(q);
            }
            catch (SQLException se) {
    			System.out.println("Error: ManageIngredientsWindow::deleteIngredient (2)");
    			se.printStackTrace();
    		}
            
            // Then, delete from the combobox.
            ingredientBox.removeItem(ingredientBox.getSelectedItem());
            
            list.update();
        } else if (n == JOptionPane.NO_OPTION) {
            // DEBUG System.out.println("No, don't delete");
        } else {
            // DEBUG System.out.println("Nothing chosen.");
        }
	}
	
	/**
	 * This renames the selected ingredient both in the database
	 * and on the window.  A dialog pops up that prompts the user
	 * for a new name, and if OK is pressed, the currently selected
	 * ingredient is renamed to the given name.
	 */
	private void renameIngredient(){
		 String inputValue = JOptionPane.showInputDialog("Enter the new name for " + ingredientBox.getSelectedItem());
		 Ingredient ingredient = (Ingredient)ingredientBox.getSelectedItem();
		 if (inputValue != null){
		 	try{
		 		// The database doesn't like apostrophes.
				String cleanName = inputValue;
				cleanName = cleanName.replaceAll("'", "''");
		 		
		 		// First check to see if there is already an Ingredient with the
				// same name.
				String q = "SELECT Name FROM Ingredient WHERE Name = '" + cleanName + "'";
				ResultSet rs = database.query(q);
				if (rs.next()){
					JOptionPane.showMessageDialog(mainPanel,
	    				    "Error: Ingredient " + rs.getString(1) + " already exists.\n" +
							"Please choose another name.",
	    				    "eShopper Error",
	    				    JOptionPane.ERROR_MESSAGE);
	    			return;
				}
				
				// If not, rename the ingredient in the database.
            	q = "UPDATE Ingredient SET Name = '" + cleanName + "' WHERE ID = " 
            		+ ingredient.getID();
            	database.update(q);
            }
            catch (SQLException se) {
    			System.out.println("Error: ManageIngredientsWindow::renameIngredient");
    			se.printStackTrace();
    		}
		 			 	
		 	// Then, rename in the comboBox by deleting the old
		 	// name, then inserting the new one.
		 	int n = ingredientBox.getSelectedIndex();
		 	ingredientBox.removeItemAt(n);
		 	ingredientBox.insertItemAt(new Ingredient(ingredient.getID(), inputValue), n);
		 	ingredientBox.setSelectedIndex(n);
		 	// ingredientBox.getSelectedItem() = inputValue;
		 }
	}
	
	/**
	 * This handles all of the ActionEvents generated by the buttons in the window.
	 */
	public void actionPerformed(ActionEvent e){
		super.actionPerformed(e);
		if (e.getActionCommand().equals("Create New"))
			displayCreateIngredientWindow();
		else if (e.getActionCommand().equals("ingredientBox"))
			getDescription((Ingredient)ingredientBox.getSelectedItem());
		else if (e.getActionCommand().equals("delete")){
			deleteIngredient();
		} else if (e.getActionCommand().equals("rename")){
			renameIngredient();
		}
	}
}
