/*
 * Created on May 12, 2004
 *
 */
package eShopper;

import java.sql.*;

/**
 * This is a class that can represent any undoable action in
 * eShopper.  When actions are performed with eShopper, an
 * instance of this class is created and added to the undo
 * buffer.
 * 
 * @author Adam
 */
public class eShopperAction {
	// Used by all action types
	int actionPerformed;
	
	// Used by Clear List
	int clearedID, newID;
	String clearedListName, clearedListOwner;
	
	// Used by Add/Remove Item
	String itemDate;
	eShopperNode myNode;
	
	public eShopperAction(){
		actionPerformed = 0;
		itemDate = "0000-00-00";
		clearedID = 0;
	}
	
	/**
	 * This allows the type of action to be set.  The available
	 * types are:
	 * 1: Clear List
	 * 2: Add Item
	 * 3: Remove Item 
	 * 
	 * @param action The type of action.
	 * @param list The list upon which the action was performed.
	 */
	public void setAction(int action, eShopperList list){
		actionPerformed = action;
		switch(actionPerformed){
			// Clear List
			case 1:
				clearedID = list.getListID();
				clearedListName = list.getListName();
				clearedListOwner = list.getListOwner();
				newID = -1;
				try{
					// Get the lowest existing listID
					String q = "SELECT TOP 1 ID FROM List ORDER BY ID ASC";
					ResultSet rs = list.getDB().query(q);
					if (rs.next()){
						newID = rs.getInt(1) - 1;
					}
					if (newID == 0)
						newID = -1;
					
					// First, copy the list ID into the "Hidden Undo Lists"
					// range ( ID < 0 ).
					q = "INSERT INTO List VALUES('" + newID + "', SELECT Owner FROM List where ID = '" + clearedID + "', '" + list.getListName() + "')";
					System.out.println(q); //DEBUG
					list.getDB().update(q);
					
					// Then, copy all of the recipes and ingredients over
					// to the hidden list.
					q = "SELECT * FROM RecipeList WHERE ListID = '" + clearedID + "'";
					System.out.println(q); //DEBUG
					rs = list.getDB().query(q);
					while (rs.next()){
						String myDate = rs.getString("Date");
						if (myDate != null)
							myDate = "'" + myDate + "'";
						String myMeal = rs.getString("Meal");
						if (myMeal != null)
							myMeal = "'" + myMeal + "'";
						q = "INSERT INTO RecipeList VALUES(" +
							"null," + 
							rs.getString("RecipeID") + "," + 
							newID + "," + 
							myMeal + "," + 
							myDate + ")";
						System.out.println(q); //DEBUG
						list.getDB().update(q);
					}
					
					q = "SELECT * FROM IngredientList WHERE ListID = '" + clearedID + "'";
					System.out.println(q); //DEBUG
					rs = list.getDB().query(q);
					while (rs.next()){
						String myMeal = rs.getString("Meal");
						if (myMeal != null)
							myMeal = "'" + myMeal + "'";
						String myDate = rs.getString("Date");
						if (myDate != null)
							myDate = "'" + myDate + "'";
						String myAmount = rs.getString("Amount");
						if (myAmount != null)
							myAmount = "'" + myAmount + "'";
						q = "INSERT INTO IngredientList VALUES(" +
							"null," + 
							rs.getString("IngredientID") + "," + 
							newID + ", " + 
							myMeal + "," +
							myDate + "," +
							myAmount + "," +
							rs.getString("UnitID") + ")";
						System.out.println(q); //DEBUG
						list.getDB().update(q);
					}
				}
				catch (SQLException se){
					System.out.println("Error: eShopperAction::setAction(1)");
					se.printStackTrace();
				}
				break;
			
			// Add Item
			case 2:
				
				break;
				
			// Remove Item
			case 3:
				
				break;
		}
	}
	
	/**
	 * This returns the type of action.  The types that can be
	 * returned are:
	 * 1: Clear List
	 * 2: Add Item
	 * 3: Remove Item
	 * 
	 * @return An integer representing the type of action performed.
	 */
	public int getAction(){
		return actionPerformed;
	}
	
	/**
	 * This reverses the affects of the action.
	 *
	 */
	public void undoAction(eShopperList list){
		if (clearedID == 0)
			return;
		switch(actionPerformed){
			// Clear List
			case 1:
				try{
					// First, see if the cleared list already exists.  If it doesn't,
					// create it.
					String q = "SELECT ID FROM List WHERE ID = " + clearedID;
					ResultSet rs = list.getDB().query(q);
					
					if (!rs.next()){
						q = "INSERT INTO List VALUES('" + clearedID + "', '" + clearedListOwner + "', '" + clearedListName + "')";
						System.out.println(q); //DEBUG
						list.getDB().update(q);
					}
					
					// Then, copy all of the recipes and ingredients over
					// from the hidden list.
					q = "SELECT * FROM RecipeList WHERE ListID = '" + newID + "'";
					System.out.println(q); //DEBUG
					rs = list.getDB().query(q);
					while (rs.next()){
						String myDate = rs.getString("Date");
						if (myDate != null)
							myDate = "'" + myDate + "'";
						String myMeal = rs.getString("Meal");
						if (myMeal != null)
							myMeal = "'" + myMeal + "'";
						q = "INSERT INTO RecipeList VALUES(" +
							"null," + 
							rs.getString("RecipeID") + "," + 
							clearedID + "," + 
							myMeal + "," + 
							myDate + ")";
						System.out.println(q); //DEBUG
						list.getDB().update(q);
					}
					
					q = "SELECT * FROM IngredientList WHERE ListID = '" + newID + "'";
					System.out.println(q); //DEBUG
					rs = list.getDB().query(q);
					while (rs.next()){
						String myMeal = rs.getString("Meal");
						if (myMeal != null)
							myMeal = "'" + myMeal + "'";
						String myDate = rs.getString("Date");
						if (myDate != null)
							myDate = "'" + myDate + "'";
						String myAmount = rs.getString("Amount");
						if (myAmount != null)
							myAmount = "'" + myAmount + "'";
						q = "INSERT INTO IngredientList VALUES(" +
							"null," + 
							rs.getString("IngredientID") + "," + 
							clearedID + "," + 
							myMeal + "," +
							myDate + "," +
							myAmount + "," +
							rs.getString("UnitID") + ")";
						System.out.println(q); //DEBUG
						list.getDB().update(q);
					}
					list.update();
				}
				catch (SQLException se){
					System.out.println("Error: eShopperAction::undoAction(1)");
					se.printStackTrace();
				}
				break;
			
			// Add Item
			case 2:
				
				break;
				
			// Remove Item
			case 3:
				
				break;
		}
	}
	
	/**
	 * This sets the node the action was performed on (if applicable).  
	 * 
	 * @param node The eShopper node upon which the action was performed.
	 */
	public void setNode(eShopperNode node){
		// Make sure the action has a node associated with it.
		if (actionPerformed > 2){
			myNode = node;
		}
	}
	
	/**
	 * This returns the node the action was performed on (if applicable).
	 * 
	 * @return The node the action was performed on, if it exists; otherwise,
	 * null.
	 */
	public eShopperNode getNode(){
		// Make sure the action has a node associated with it.
		if (actionPerformed > 2){
			return myNode;
		}else{
			return null;
		}
	}
	
	/**
	 * This sets the date associated with the Action's node (if applicable).
	 * 
	 * @param date The date associated with the Action's node.
	 */
	public void setDate(String date){
		// Make sure the action has a date associated with it.
		if (actionPerformed > 2){
			itemDate = date;
		}
	}
	
	/**
	 * This returns the date associated with the Action's node (if applicable).
	 *
	 * @return The date associated with the Action's node, if it exists;
	 * otherwise, null.
	 */
	public String getDate(){
		// Make sure the action has a date associated with it.
		if (actionPerformed > 2){
			return itemDate;
		} else {
			return null;
		}
	}
	
}
