package eShopper;

import java.sql.*;
import java.text.SimpleDateFormat;
import java.util.Vector;

/*
 * Created on Apr 25, 2004
 * 
 */

/**
 * This is a subclass of eShopperNode which represents a date node.
 * A date node can have both recipes and ingredients as children.
 * 
 * @author Jim Laing
 */
public class eShopperDate extends eShopperNode {
	
	Date date;
	private Vector children;

	/**
	 * the constructor
	 * @param date a java.sql.date object
	 * @param listID the database ID of the list which the date is associated with
	 * @param db the database from which to build the node
	 */
	public eShopperDate(Date date, int listID, eShopperDB db) {
		this.date = date;
		this.listID = listID;
		this.db = db;
		this.children = getItemVector();
	}
	
	/**
	 * This returns the node's children.
	 * 
	 * @return The childen of the node
	 */
	public Vector getChildren() {
		return children;		
	}

	/**
	 * This is the overload of the toString function.
	 * 
	 * @return a string representation of the node
	 */
	public String toString() {
		SimpleDateFormat textDate = new SimpleDateFormat("EEEE, MMMM d yyyy");
		return textDate.format(date);
	}
	
	/**
	 * This returns the date associated with the node.
	 * 
	 * @return a string representation of the date
	 */
	public String getDate() {
		return date.toString();
	}
	
	/**
	 * This returns the nodeID of the node.
	 * 
	 * @return always returns 0, which is the nodeID of any date
	 */
	public int nodeID() {
		// All eShopperDate nodes have a nodeID of 0.
		return 0;
	}
	
	/**
	 * Gets a string representing the date, formatted to go in a database (e.g. '2004-07-04') 
	 *
	 * @return the string representation of the date
	 */
	public String getDBString() {
		SimpleDateFormat yyyymmdd = new SimpleDateFormat("''yyyy-MM-dd''");
		return yyyymmdd.format(date);
	}
	
	/**
	 * This returns the children of this date node, both recipes and ingredients
	 * 
	 * @return a vector represention the items associated with this date
	 */
	private Vector getItemVector() {
			Vector v = new Vector();
			String q = new String();
		
			try {
						// get all the recipes for an associated date
						q = "SELECT ID, RECIPEID FROM RECIPELIST WHERE DATE = '" + date + "' " +
							"AND LISTID = " + listID;
						ResultSet rs = db.query(q);	
						while(rs.next()) 
							v.add(new eShopperRecipe(rs.getInt("ID"), rs.getInt("RECIPEID"), listID, db)); 
			
						// get all the ingredients for the associated date
						q = "SELECT ID, INGREDIENTID, AMOUNT, UNITID FROM INGREDIENTLIST WHERE DATE = '" 
							+ date + "' AND LISTID = " + listID;
						rs = db.query(q);	
						while(rs.next()) 
							v.add(new eShopperIngredient(rs.getInt("ID"), rs.getInt("INGREDIENTID"), 
								listID, db, rs.getFloat("AMOUNT"), rs.getInt("UNITID"))); 
			
						return v;
					} catch (SQLException e) {
						System.out.println("Error: eShopperDate::getIngredients");
						e.printStackTrace();
					}
					return null; 
		}
}